@@ -1,4 +1,6 @@
 
+#include "jackalope_config.h" // controls debugging and diagnostics output
+
 #include <RcppArmadillo.h>
 #include <algorithm> // lower_bound
 #include <vector>  // vector class
@@ -12,12 +14,14 @@
 #endif
 
 #include "jackalope_types.h"  // integer types
-#include "seq_classes_ref.h"  // Ref* classes
-#include "seq_classes_var.h"  // Var* classes
+#include "ref_classes.h"  // Ref* classes
+#include "var_classes.h"  // Var* classes
 #include "hts.h"  // generic sequencing classes
 #include "hts_illumina.h"  // Illumina-specific classes
 #include "str_manip.h"  // rev_comp
 #include "io.h"  // File* types
+#include "util.h"  // split_int
+
 
 using namespace Rcpp;
 
@@ -25,16 +29,22 @@ using namespace Rcpp;
 
 
 
-// Sample one set of read strings (each with 4 lines: ID, sequence, "+", quality)
+// Sample one set of read strings (each with 4 lines: ID, chromosome, "+", quality)
 template <typename T>
 template <typename U>
 void IlluminaOneGenome<T>::one_read(std::vector<U>& fastq_pools,
+                                    bool& finished,
                                     pcg64& eng) {
 
     /*
-     Sample fragment info, and set the sequence space(s) required for these read(s).
+     Sample fragment info, and set the chromosome space(s) required for these read(s).
      */
-    seq_indels_frag(eng);
+    chrom_indels_frag(eng);
+
+    if (constr_info.chrom_ind == chromosomes->size())  {
+        finished = true;
+        return;
+    }
 
     // Fill the reads and qualities
     append_pools<U>(fastq_pools, eng);
@@ -42,6 +52,27 @@ void IlluminaOneGenome<T>::one_read(std::vector<U>& fastq_pools,
     return;
 }
 
+// Overloaded for when we input a variant chromosome stored as string
+template <typename T>
+template <typename U>
+void IlluminaOneGenome<T>::one_read(const std::string& chrom,
+                                    const uint64& chrom_i,
+                                    std::vector<U>& fastq_pools,
+                                    pcg64& eng) {
+
+    constr_info.chrom_ind = chrom_i;
+
+    /*
+     Sample indels, and set the chromosome space(s) required for these read(s).
+     */
+    indels_frag(eng);
+
+    // Fill the reads and qualities
+    append_pools<U>(chrom, fastq_pools, eng);
+
+    return;
+}
+
 /*
  Same as above, but for a duplicate. It's assumed that `one_read` has been
  run once before.
@@ -49,6 +80,7 @@ void IlluminaOneGenome<T>::one_read(std::vector<U>& fastq_pools,
 template <typename T>
 template <typename U>
 void IlluminaOneGenome<T>::re_read(std::vector<U>& fastq_pools,
+                                   bool& finished,
                                    pcg64& eng) {
 
     // Here I'm just re-doing indels bc it's a duplicate.
@@ -59,37 +91,25 @@ void IlluminaOneGenome<T>::re_read(std::vector<U>& fastq_pools,
 
     return;
 }
-
-/*
- Add information about a RefGenome or VarGenome object
- This is used when making multiple samplers that share most info except for
- that related to the sequence object.
- */
 template <typename T>
-void IlluminaOneGenome<T>::add_seq_info(const T& seq_object, const std::string& barcode) {
+template <typename U>
+void IlluminaOneGenome<T>::re_read(const std::string& chrom,
+                                   const uint64& chrom_i,
+                                   std::vector<U>& fastq_pools,
+                                   pcg64& eng) {
 
-    seq_lengths = seq_object.seq_sizes();
-    sequences = &seq_object;
-    name = seq_object.name;
-    constr_info.barcode = barcode;
+    constr_info.chrom_ind = chrom_i;
 
-    construct_seqs();
+    // Here I'm just re-doing indels bc it's a duplicate.
+    just_indels(eng);
+
+    // Fill the reads and qualities
+    append_pools<U>(chrom, fastq_pools, eng);
 
     return;
 }
 
 
-// Construct sequence-sampling probabilities:
-template <typename T>
-void IlluminaOneGenome<T>::construct_seqs() {
-    std::vector<double> probs_;
-    probs_.reserve(seq_lengths.size());
-    for (uint64 i = 0; i < seq_lengths.size(); i++) {
-        probs_.push_back(static_cast<double>(seq_lengths[i]));
-    }
-    seq_sampler = AliasSampler(probs_);
-    return;
-}
 
 
 // Sample for insertion and deletion positions
@@ -129,18 +149,18 @@ void IlluminaOneGenome<T>::sample_indels(pcg64& eng) {
     return;
 }
 
-// Adjust sequence spaces
+// Adjust chromosome spaces
 template <typename T>
-void IlluminaOneGenome<T>::adjust_seq_spaces() {
+void IlluminaOneGenome<T>::adjust_chrom_spaces() {
 
-    std::vector<uint64>& read_seq_spaces(constr_info.read_seq_spaces);
+    std::vector<uint64>& read_chrom_spaces(constr_info.read_chrom_spaces);
     std::vector<std::string>& reads(constr_info.reads);
     const uint64& frag_len(constr_info.frag_len);
 
     for (uint64 r = 0; r < insertions.size(); r++) {
         /*
          I'm adding deletions because more deletions mean that I need
-         more sequence bases to achieve the same read length.
+         more chromosome bases to achieve the same read length.
          Insertions means I need fewer.
          */
         sint64 indel_effect = static_cast<sint64>(deletions[r].size()) -
@@ -151,13 +171,13 @@ void IlluminaOneGenome<T>::adjust_seq_spaces() {
          (Because the indel sampler stops when it reaches the fragment end,
          we don't need to account for that.)
          */
-        read_seq_spaces[r] = std::min(read_length + indel_effect, frag_len);
-        // Adjust `reads` so it can hold the necessary sequence space:
-        if (reads[r].size() != read_seq_spaces[r]) {
-            reads[r].resize(read_seq_spaces[r], 'N');
+        read_chrom_spaces[r] = std::min(read_length + indel_effect, frag_len);
+        // Adjust `reads` so it can hold the necessary chromosome space:
+        if (reads[r].size() != read_chrom_spaces[r]) {
+            reads[r].resize(read_chrom_spaces[r], 'N');
         }
         // Now including effect of barcode:
-        read_seq_spaces[r] -= constr_info.barcode.size();
+        read_chrom_spaces[r] -= constr_info.barcode.size();
     }
 
     return;
@@ -165,19 +185,22 @@ void IlluminaOneGenome<T>::adjust_seq_spaces() {
 
 
 /*
- Sample a sequence, indels, fragment length, and starting position for the fragment.
- Lastly, it sets the sequence spaces required for these reads.
+ Sample a chromosome, indels, fragment length, and starting position for the fragment.
+ Lastly, it sets the chromosome spaces required for these reads.
  */
 template <typename T>
-void IlluminaOneGenome<T>::seq_indels_frag(pcg64& eng) {
+void IlluminaOneGenome<T>::chrom_indels_frag(pcg64& eng) {
 
-    uint64& seq_ind(constr_info.seq_ind);
+    uint64& chrom_ind(constr_info.chrom_ind);
     uint64& frag_len(constr_info.frag_len);
     uint64& frag_start(constr_info.frag_start);
 
-    // Sample sequence:
-    seq_ind = seq_sampler.sample(eng);
-    uint64 seq_len = (*sequences)[seq_ind].size();
+    // Get chromosome:
+    chrom_ind = 0;
+    while (chrom_ind < chrom_reads.size() && chrom_reads[chrom_ind] == 0) chrom_ind++;
+    if (chrom_ind == chromosomes->size()) return;
+
+    uint64 chrom_len = (*chromosomes)[chrom_ind].size();
 
     // Sample fragment length:
     frag_len = static_cast<uint64>(frag_lengths(eng));
@@ -185,27 +208,66 @@ void IlluminaOneGenome<T>::seq_indels_frag(pcg64& eng) {
     if (frag_len > frag_len_max) frag_len = frag_len_max;
 
     // Sample fragment starting position:
-    if (frag_len >= seq_len) {
-        frag_len = seq_len;
+    if (frag_len >= chrom_len) {
+        frag_len = chrom_len;
         frag_start = 0;
     } else {
         double u = runif_01(eng);
-        frag_start = static_cast<uint64>(u * (seq_len - frag_len + 1));
+        frag_start = static_cast<uint64>(u * (chrom_len - frag_len + 1));
     }
 
     // Sample indels:
     sample_indels(eng);
 
-    // Adjust sequence spaces:
-    adjust_seq_spaces();
+    // Adjust chromosome spaces:
+    adjust_chrom_spaces();
+
+    return;
+}
+
+
+
+/*
+ Sample indels, fragment length, and starting position for the fragment.
+ Lastly, it sets the chromosome spaces required for these reads.
+ This is for when the chromosome is already set.
+ */
+template <typename T>
+void IlluminaOneGenome<T>::indels_frag(pcg64& eng) {
+
+    uint64& chrom_ind(constr_info.chrom_ind);
+    uint64& frag_len(constr_info.frag_len);
+    uint64& frag_start(constr_info.frag_start);
+
+    uint64 chrom_len = (*chromosomes)[chrom_ind].size();
+
+    // Sample fragment length:
+    frag_len = static_cast<uint64>(frag_lengths(eng));
+    if (frag_len < frag_len_min) frag_len = frag_len_min;
+    if (frag_len > frag_len_max) frag_len = frag_len_max;
+
+    // Sample fragment starting position:
+    if (frag_len >= chrom_len) {
+        frag_len = chrom_len;
+        frag_start = 0;
+    } else {
+        double u = runif_01(eng);
+        frag_start = static_cast<uint64>(u * (chrom_len - frag_len + 1));
+    }
+
+    // Sample indels:
+    sample_indels(eng);
+
+    // Adjust chromosome spaces:
+    adjust_chrom_spaces();
 
     return;
 }
 
 
 /*
- Same as above, but for duplicates.
- This means skipping the sequence and fragment info parts.
+ Same as `chrom_indels_frag` above, but for duplicates.
+ This means skipping the chromosome and fragment info parts.
  */
 template <typename T>
 void IlluminaOneGenome<T>::just_indels(pcg64& eng) {
@@ -213,16 +275,61 @@ void IlluminaOneGenome<T>::just_indels(pcg64& eng) {
     // Sample indels:
     sample_indels(eng);
 
-    // Adjust sequence spaces:
-    adjust_seq_spaces();
+    // Adjust chromosome spaces:
+    adjust_chrom_spaces();
 
     return;
 }
 
 
+template <typename U>
+void fill_fq_lines(U& fq_pool,
+                   const std::string& name,
+                   const std::string& chrom_name,
+                   const std::string& read,
+                   const std::string& qual,
+                   const uint64& i,
+                   const uint64& start,
+                   const bool& paired,
+                   bool& reverse) {
+
+    // Combine into 4 lines of output per read:
+    // ID line:
+    fq_pool.push_back('@');
+    for (const char& c : name) fq_pool.push_back(c);
+    fq_pool.push_back('-');
+    for (const char& c : chrom_name) fq_pool.push_back(c);
+    fq_pool.push_back('-');
+    for (const char& c : std::to_string(start)) fq_pool.push_back(c);
+    fq_pool.push_back('-');
+    if (reverse) {
+        fq_pool.push_back('R');
+    } else fq_pool.push_back('F');
+    if (paired) {
+        fq_pool.push_back('/');
+        for (const char& c : std::to_string(i+1)) fq_pool.push_back(c);
+    }
+    fq_pool.push_back('\n');
+    // The rest:
+    for (const char& c : read) fq_pool.push_back(c);
+    fq_pool.push_back('\n');
+    fq_pool.push_back('+');
+    fq_pool.push_back('\n');
+    for (const char& c : qual) fq_pool.push_back(c);
+    fq_pool.push_back('\n');
+
+    // If doing paired reads, the second one should be the reverse of the first
+    reverse = !reverse;
+
+    return;
+
+}
+
+
+
 
 /*
- Sample one set of read strings (each with 4 lines: ID, sequence, "+", quality),
+ Sample one set of read strings (each with 4 lines: ID, chromosome, "+", quality),
  then append that to the `fastq_pools` vector.
  This function does NOT do anything with fragments.
  That should be done outside this function.
@@ -236,10 +343,10 @@ void IlluminaOneGenome<T>::append_pools(std::vector<U>& fastq_pools,
     if (fastq_pools.size() != n_read_ends) fastq_pools.resize(n_read_ends);
 
     const std::string& barcode(constr_info.barcode);
-    const std::vector<uint64>& read_seq_spaces(constr_info.read_seq_spaces);
+    const std::vector<uint64>& read_chrom_spaces(constr_info.read_chrom_spaces);
 
     // Just making this reference to keep lines from getting very long.
-    const uint64& seq_ind(constr_info.seq_ind);
+    const uint64& chrom_ind(constr_info.chrom_ind);
 
     // Boolean for whether we take the reverse side first:
     bool reverse = runif_01(eng) < 0.5;
@@ -253,7 +360,7 @@ void IlluminaOneGenome<T>::append_pools(std::vector<U>& fastq_pools,
             start = constr_info.frag_start;
         } else {
             start = constr_info.frag_start + constr_info.frag_len -
-                read_seq_spaces[i];
+                read_chrom_spaces[i];
         }
 
         /*
@@ -261,21 +368,21 @@ void IlluminaOneGenome<T>::append_pools(std::vector<U>& fastq_pools,
         */
         if (!reverse) {
             // Fill in read starting with position after barcode:
-            (*(sequences))[seq_ind].fill_read(
+            (*(chromosomes))[chrom_ind].fill_read(
                     read, barcode.size(),
-                    start, read_seq_spaces[i]);
+                    start, read_chrom_spaces[i]);
         } else {
             /*
-             If doing reverse, we can add the actual sequence to the front of the
+             If doing reverse, we can add the actual chromosome to the front of the
              read instead of starting at `barcode.size()`.
 
              (Even though `rev_comp` requires only T, C, A, G, or N characters,
              `read` should already have filler 'N' chars present at initialization
              of the `constr_info` field, so no need to add any now.)
              */
-            (*(sequences))[seq_ind].fill_read(
+            (*(chromosomes))[chrom_ind].fill_read(
                     read, 0,
-                    start, read_seq_spaces[i]);
+                    start, read_chrom_spaces[i]);
             // Now do reverse complement:
             rev_comp(read);
         }
@@ -286,33 +393,88 @@ void IlluminaOneGenome<T>::append_pools(std::vector<U>& fastq_pools,
         // Sample mapping quality and add errors to read:
         qual_errors[i].fill_read_qual(read, qual, insertions[i], deletions[i], eng);
 
-        // Combine into 4 lines of output per read:
-        // ID line:
-        fastq_pools[i].push_back('@');
-        for (const char& c : this->name) fastq_pools[i].push_back(c);
-        fastq_pools[i].push_back('-');
-        for (const char& c : (*sequences)[seq_ind].name) fastq_pools[i].push_back(c);
-        fastq_pools[i].push_back('-');
-        for (const char& c : std::to_string(start)) fastq_pools[i].push_back(c);
-        fastq_pools[i].push_back('-');
-        if (reverse) {
-            fastq_pools[i].push_back('R');
-        } else fastq_pools[i].push_back('F');
-        if (paired) {
-            fastq_pools[i].push_back('/');
-            for (const char& c : std::to_string(i+1)) fastq_pools[i].push_back(c);
-        }
-        fastq_pools[i].push_back('\n');
-        // The rest:
-        for (const char& c : read) fastq_pools[i].push_back(c);
-        fastq_pools[i].push_back('\n');
-        fastq_pools[i].push_back('+');
-        fastq_pools[i].push_back('\n');
-        for (const char& c : qual) fastq_pools[i].push_back(c);
-        fastq_pools[i].push_back('\n');
+        std::string chrom_name = (*chromosomes)[chrom_ind].name;
 
-        // If doing paired reads, the second one should be the reverse of the first
-        reverse = !reverse;
+        // Combine into 4 lines of output per read, and add to `fastq_pools[i]`
+        fill_fq_lines<U>(fastq_pools[i], name, chrom_name, read, qual, i, start,
+                         paired, reverse);
+
+    }
+
+    if (chrom_reads[constr_info.chrom_ind] < n_read_ends) {
+        chrom_reads[constr_info.chrom_ind] = 0;
+    } else chrom_reads[constr_info.chrom_ind] -= n_read_ends;
+
+    return;
+}
+
+// Overloaded for when a variant chromosome sequence is provided
+template <typename T>
+template <typename U>
+void IlluminaOneGenome<T>::append_pools(const std::string& chrom,
+                                        std::vector<U>& fastq_pools,
+                                        pcg64& eng) {
+
+    uint64 n_read_ends = ins_probs.size();
+    if (fastq_pools.size() != n_read_ends) fastq_pools.resize(n_read_ends);
+
+    const std::string& barcode(constr_info.barcode);
+    const std::vector<uint64>& read_chrom_spaces(constr_info.read_chrom_spaces);
+
+    // Just making this reference to keep lines from getting very long.
+    const uint64& chrom_ind(constr_info.chrom_ind);
+
+    // Boolean for whether we take the reverse side first:
+    bool reverse = runif_01(eng) < 0.5;
+    for (uint64 i = 0; i < n_read_ends; i++) {
+        std::string& read(constr_info.reads[i]);
+        std::string& qual(constr_info.quals[i]);
+
+        // Read starting location depends on if mate-pair and if reverse strand:
+        uint64 start;
+        if ((!matepair && !reverse) || (matepair && reverse)) {
+            start = constr_info.frag_start;
+        } else {
+            start = constr_info.frag_start + constr_info.frag_len -
+                read_chrom_spaces[i];
+        }
+
+        /*
+         Filling `read` only depends on whether it's the reverse strand:
+        */
+        if (!reverse) {
+
+            // Fill in read starting with position after barcode:
+            fill_read__(chrom, read, barcode.size(), start, read_chrom_spaces[i]);
+            // `fill_read__` fxn found in hts.h
+
+
+        } else {
+            /*
+             If doing reverse, we can add the actual chromosome to the front of the
+             read instead of starting at `barcode.size()`.
+
+             (Even though `rev_comp` requires only T, C, A, G, or N characters,
+             `read` should already have filler 'N' chars present at initialization
+             of the `constr_info` field, so no need to add any now.)
+             */
+            fill_read__(chrom, read, 0, start, read_chrom_spaces[i]);
+
+            // Now do reverse complement:
+            rev_comp(read);
+        }
+
+        // Now fill barcode:
+        for (uint64 i = 0; i < barcode.size(); i++) read[i] = barcode[i];
+
+        // Sample mapping quality and add errors to read:
+        qual_errors[i].fill_read_qual(read, qual, insertions[i], deletions[i], eng);
+
+        std::string chrom_name = (*chromosomes)[chrom_ind].name;
+
+        // Combine into 4 lines of output per read, and add to `fastq_pools[i]`
+        fill_fq_lines<U>(fastq_pools[i], name, chrom_name, read, qual, i, start,
+                         paired, reverse);
 
     }
 
@@ -325,6 +487,81 @@ void IlluminaOneGenome<T>::append_pools(std::vector<U>& fastq_pools,
 
 
 
+
+
+
+
+// If only providing rng and id info, sample for a variant, then make read(s):
+template <typename U>
+void IlluminaVariants::one_read(std::vector<U>& fastq_pools,
+                                bool& finished,
+                                pcg64& eng) {
+
+    if (var == variants->size()) {
+        finished = true;
+        return;
+    }
+
+    if (n_reads_vc[var][chr] == 0 || var_chrom_seq.empty()) {
+
+        uint64 new_var = var;
+        uint64 new_chr = chr;
+        for (; new_var < n_reads_vc.size(); new_var++) {
+            while (n_reads_vc[new_var][new_chr] == 0) {
+                new_chr++;
+                if (new_chr == n_reads_vc[new_var].size()) break;
+            }
+            if (new_chr < n_reads_vc[new_var].size()) {
+                break;
+            } else new_chr = 0;
+        }
+
+        var = new_var;
+        chr = new_chr;
+
+        if (var == variants->size())  {
+            finished = true;
+            return;
+        }
+
+        var_chrom_seq = (*variants)[var][chr].get_chrom_full();
+    }
+
+    read_makers[var].one_read<U>(var_chrom_seq, chr, fastq_pools, eng);
+
+    n_reads_vc[var][chr]--;
+    if (paired && n_reads_vc[var][chr] > 0) n_reads_vc[var][chr]--;
+
+    return;
+}
+/*
+ -------------
+ `re_read` methods (for duplicates)
+ -------------
+ */
+template <typename U>
+void IlluminaVariants::re_read(std::vector<U>& fastq_pools,
+                               bool& finished,
+                               pcg64& eng) {
+
+    if (var == variants->size()) {
+        finished = true;
+        return;
+    }
+
+    read_makers[var].re_read<U>(var_chrom_seq, chr, fastq_pools, eng);
+
+    if (n_reads_vc[var][chr] > 0) n_reads_vc[var][chr]--;
+    if (paired && n_reads_vc[var][chr] > 0) n_reads_vc[var][chr]--;
+
+    return;
+}
+
+
+
+
+
+
 /*
  ========================================================================================
  ========================================================================================
@@ -343,7 +580,7 @@ void IlluminaOneGenome<T>::append_pools(std::vector<U>& fastq_pools,
 
 
 
-//' Illumina sequence for reference object.
+//' Illumina chromosome for reference object.
 //'
 //'
 //' @noRd
@@ -375,6 +612,7 @@ void illumina_ref_cpp(SEXP ref_genome_ptr,
                       const std::vector<std::string>& barcodes) {
 
     XPtr<RefGenome> ref_genome(ref_genome_ptr);
+
     IlluminaReference read_filler_base;
 
     uint64 n_read_ends;
@@ -415,7 +653,7 @@ void illumina_ref_cpp(SEXP ref_genome_ptr,
 
 
 
-//' Illumina sequence for reference object.
+//' Illumina chromosome for reference object.
 //'
 //'
 //' @noRd
@@ -452,7 +690,9 @@ void illumina_var_cpp(SEXP var_set_ptr,
     IlluminaVariants read_filler_base;
 
     uint64 n_read_ends;
+
     if (paired) {
+
         n_read_ends = 2;
         read_filler_base = IlluminaVariants(*var_set, variant_probs,
                                             matepair,
@@ -461,13 +701,16 @@ void illumina_var_cpp(SEXP var_set_ptr,
                                             qual_probs1, quals1, ins_prob1, del_prob1,
                                             qual_probs2, quals2, ins_prob2, del_prob2,
                                             barcodes);
+
     } else {
+
         n_read_ends = 1;
         read_filler_base = IlluminaVariants(*var_set, variant_probs,
                                             frag_len_shape, frag_len_scale,
                                             frag_len_min, frag_len_max,
                                             qual_probs1, quals1, ins_prob1, del_prob1,
                                             barcodes);
+
     }
 
     // For doing multithreaded compression after initial uncompressed run:
